==> Texas-0.1.3.crate: Texas-0.1.3/src/document.rs <==
use std::{collections::HashMap, fmt::Display};

use crate::{commands::Command, component::Component, errors::TexError, AsLatex, Opt, Res};

pub enum DocumentClassType {
    Article,
    Part,
    Report,
    Book,
}
impl Display for DocumentClassType {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(
            f,
            "{}",
            match &self {
                Self::Article => "article",
                Self::Part => "part",
                Self::Report => "report",
                Self::Book => "book",
            }
        )?;

        Ok(())
    }
}
impl From<&str> for DocumentClassType {
    fn from(value: &str) -> Self {
        match value {
            "article" => Self::Article,
            "part" => Self::Part,
            "book" => Self::Book,
            "report" => Self::Report,
            _ => Self::Article,
        }
    }
}

pub struct DocumentClass {
    typ: DocumentClassType,
    opt: Vec<String>,
}

impl DocumentClass {
    pub fn new(typ: &str) -> Self {
        Self {
            typ: typ.into(),
            opt: vec![],
        }
    }
}
impl AsLatex for DocumentClass {
    fn to_string(&self) -> String {
        let options = self
            .opt
            .iter()
            .map(|s| format!("{}, ", s))
            .collect::<String>();
        format!("\\documentclass[{}]{{{}}}", options, self.typ.to_string())
    }
}
impl Opt for DocumentClass {
    fn add_option(&mut self, opt: &str) {
        self.opt.push(opt.to_string())
    }
}

pub struct Package {
    name: String,
    opt: Vec<String>,
}

impl AsLatex for Package {
    fn to_string(&self) -> String {
        let options = self
            .opt
            .iter()
            .map(|s| format!("{}, ", s))
            .collect::<String>();
        format!("\\usepackage[{}]{{{}}}\n", options, self.name)
    }
}
impl Opt for Package {
    fn add_option(&mut self, opt: &str) {
        self.opt.push(opt.to_string())
    }
}
impl Package {
    pub fn new(name: &str) -> Self {
        Self {
            name: name.to_string(),
            opt: vec![],
        }
    }
}

pub struct Metadata {
    title: String,
    author: String,
}
impl AsLatex for Metadata {
    fn to_string(&self) -> String {
        format!(
            "\\title{{{}}}\n\\author{{{}}}\n\\maketitle\n",
            self.title, self.author
        )
    }
}
impl Metadata {
    pub fn new(title: String, author: String) -> Self {
        Self { title, author }
    }
}

pub struct Document {
    document_class: DocumentClass,
    packages: Vec<Package>,
    metadata: Metadata,
    components: Vec<Component>,
    commands: HashMap<String, Command>,
}
impl AsLatex for Document {
    fn to_string(&self) -> String {
        let dc = self.document_class.to_string();
        let pkgs = self
            .packages
            .iter()
            .map(|x| x.to_string())
            .collect::<String>();
        let md = self.metadata.to_string();
        let body = self
            .components
            .iter()
            .map(|x| x.to_string())
            .collect::<String>();
        let cmd = self
            .commands
            .iter()
            .map(|x| format!("{} \n", x.1.to_string()))
            .collect::<String>();
        format!(
            "{}\n{}\n{}\n\\begin{{document}}\n{}\n{}\n\\end{{document}}",
            dc, pkgs, cmd, md, body
        )
    }
}
impl Document {
    pub fn new(class: DocumentClass) -> Self {
        Self {
            document_class: class,
            packages: vec![],
            metadata: Metadata {
                title: "".to_string(),
                author: "".to_string(),
            },
            components: vec![],
            commands: HashMap::new(),
        }
    }

    pub fn get_command(&self, cmd: &String) -> Res<Command> {
        match self.commands.get(cmd) {
            Some(s) => Ok(s.clone()),
            None => Err(TexError::Undefined.into()),
        }
    }

    pub fn new_command(&mut self, c: Command) {
        self.commands.insert(c.name.clone(), c);
    }

    pub fn new_component(&mut self, new: Component) {
        self.components.push(new);
    }

    pub fn set_md(&mut self, title: &str, author: &str) {
        self.metadata = Metadata {
            title: title.to_string(),
            author: author.to_string(),
        };
    }

    pub fn new_package(&mut self, new: Package) {
        self.packages.push(new);
    }
}

==> Texas-0.1.5.crate: Texas-0.1.5/src/document.rs <==
use std::{collections::HashMap, fmt::Display};

use crate::{commands::Command, component::Component, errors::TexError, AsLatex, Opt, Res};

pub enum DocumentClassType {
    Article,
    Part,
    Report,
    Book,
}
impl Display for DocumentClassType {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(
            f,
            "{}",
            match &self {
                Self::Article => "article",
                Self::Part => "part",
                Self::Report => "report",
                Self::Book => "book",
            }
        )?;

        Ok(())
    }
}
impl From<&str> for DocumentClassType {
    fn from(value: &str) -> Self {
        match value {
            "article" => Self::Article,
            "part" => Self::Part,
            "book" => Self::Book,
            "report" => Self::Report,
            _ => Self::Article,
        }
    }
}

pub struct DocumentClass {
    typ: DocumentClassType,
    opt: Vec<String>,
}

impl DocumentClass {
    pub fn new(typ: &str) -> Self {
        Self {
            typ: typ.into(),
            opt: vec![],
        }
    }
}
impl AsLatex for DocumentClass {
    fn to_string(&self) -> String {
        let options = self
            .opt
            .iter()
            .map(|s| format!("{}, ", s))
            .collect::<String>();
        format!("\\documentclass[{}]{{{}}}", options, self.typ.to_string())
    }
}
impl Opt for DocumentClass {
    fn add_option(&mut self, opt: &str) {
        self.opt.push(opt.to_string())
    }
}

pub struct Package {
    name: String,
    opt: Vec<String>,
}

impl AsLatex for Package {
    fn to_string(&self) -> String {
        let options = self
            .opt
            .iter()
            .map(|s| format!("{}, ", s))
            .collect::<String>();
        format!("\\usepackage[{}]{{{}}}\n", options, self.name)
    }
}
impl Opt for Package {
    fn add_option(&mut self, opt: &str) {
        self.opt.push(opt.to_string())
    }
}
impl Package {
    pub fn new(name: &str) -> Self {
        Self {
            name: name.to_string(),
            opt: vec![],
        }
    }
}

pub struct Metadata {
    title: String,
    author: String,
}
impl AsLatex for Metadata {
    fn to_string(&self) -> String {
        format!(
            "\\title{{{}}}\n\\author{{{}}}\n\\maketitle\n",
            self.title, self.author
        )
    }
}
impl Metadata {
    pub fn new(title: String, author: String) -> Self {
        Self { title, author }
    }
}

pub struct Document {
    document_class: DocumentClass,
    packages: Vec<Package>,
    metadata: Metadata,
    components: Vec<Component>,
    commands: HashMap<String, Command>,
}
impl AsLatex for Document {
    fn to_string(&self) -> String {
        let dc = self.document_class.to_string();
        let pkgs = self
            .packages
            .iter()
            .map(|x| x.to_string())
            .collect::<String>();
        let md = self.metadata.to_string();
        let body = self
            .components
            .iter()
            .map(|x| x.to_string())
            .collect::<String>();
        let cmd = self
            .commands
            .iter()
            .map(|x| format!("{} \n", x.1.declare()))
            .collect::<String>();
        format!(
            "{}\n{}\n{}\n\\begin{{document}}\n{}\n{}\n\\end{{document}}",
            dc, pkgs, cmd, md, body
        )
    }
}
impl Document {
    pub fn new(class: DocumentClass) -> Self {
        Self {
            document_class: class,
            packages: vec![],
            metadata: Metadata {
                title: "".to_string(),
                author: "".to_string(),
            },
            components: vec![],
            commands: HashMap::new(),
        }
    }

    pub fn get_command(&self, cmd: &str) -> Res<Command> {
        match self.commands.get(cmd) {
            Some(s) => Ok(s.clone()),
            None => Err(TexError::Undefined.into()),
        }
    }

    pub fn new_command(&mut self, c: Command) {
        self.commands.insert(c.name.clone(), c);
    }

    pub fn new_component(&mut self, new: Component) {
        self.components.push(new);
    }

    pub fn set_md(&mut self, title: &str, author: &str) {
        self.metadata = Metadata {
            title: title.to_string(),
            author: author.to_string(),
        };
    }

    pub fn new_package(&mut self, new: Package) {
        self.packages.push(new);
    }
}
